import type { Context } from '../../context';
import type { MiddlewareHandler } from '../../types';
type OnError = (c: Context) => Response | Promise<Response>;
type BodyLimitOptions = {
    maxSize: number;
    onError?: OnError;
};
/**
 * Body Limit Middleware
 *
 * @example
 * ```ts
 * app.post(
 *  '/hello',
 *  bodyLimit({
 *    maxSize: 100 * 1024, // 100kb
 *    onError: (c) => {
 *      return c.text('overflow :(', 413)
 *    }
 *  }),
 *  (c) => {
 *    return c.text('pass :)')
 *  }
 * )
 * ```
 */
export declare const bodyLimit: (options: BodyLimitOptions) => MiddlewareHandler;
export {};
